﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;

using Toast.Analytics;

public class GameManager : MonoBehaviour {
	public static bool gameOver=false;
	public static bool gameStart=false;
	public static bool campaignStart=false;
	public static int score=0;

	void Start () {
		score=0;
		gameOver = false;
		gameStart = false;

		// ToastAnalytics ////////////////////////////////////////////////////////
		// setDebugMode를 true로 하는 경우 Logcat에 로그를 출력합니다. 이때 Anlaytics에서 남기는 로그는 tag를 "Analytics"로 검색하면 됩니다.
		// 또한 debug mode가 true인 경우는 analytics 웹페이지의 "개발자 웹콘솔"에도 로그가 출력됩니다. 
		// 따라서 앱 출시 전에는 debug mode를 반드시 false로 설정해야 합니다.
		GameAnalytics.setDebugMode (true);

		// ToastAnalytics ////////////////////////////////////////////////////////
		// Analytics를 초기화 합니다.
		// AppKey와 컴퍼니 아이디는 앱 설정 웹페이지에서 확인할 수 있습니다.
		// 게임 버전은 출시할 게임 버전을 사용하면 됩니다.
		// use logging user id flag는 프로그래밍 가이드 문서를 참고하여 게임 상황에 맞는 값을 사용하면 됩니다. (프로그래밍 가이드의 "사용자 구분 기준 설정"을 참고하세요)
		GameAnalytics.initializeSdk ("YOUR_APP_KEY", "YOUR_COMPANY_ID", "YOUR_APP_VERSION", false);

		// ToastAnalytics ////////////////////////////////////////////////////////
		// SDK에게 User ID를 전달합니다.
		// initializeSdk에서 use logging user id를 false로 설정 했어도 캠페인/프로모션은 User ID 기반으로 진행합니다.
		// 그리고 캠페인/프로모션을 사용하는 경우 setUserId의 useCampaignOrPromotion flag를 true로 설정해야 합니다.
		GameAnalytics.setUserId ("testuser", true);

		// ToastAnalytics ////////////////////////////////////////////////////////
		// 캠페인/프로모션을 위한 listener를 등록합니다.
		// 각 callback에 대한 내용은 Programming Guide 참고하세요.
		MyCampaignListener campaignListener = new MyCampaignListener (){};
		GameAnalytics.setCampaignListener (campaignListener);

		GameAnalytics.traceActivation ();
	}
	
	void Update () {
		if (!gameStart) {
			if(Input.GetMouseButtonDown(0)){
				gameStart = true;
				campaignStart = false;

				// ToastAnalytics ////////////////////////////////////////////////////////
				// 아이템 사용 추적 샘플입니다.
				// Programming Guide의 "재화 획득/사용"을 참고하세요.
				GameAnalytics.traceMoneyConsumption("GAME_COIN", "START_GAME", 1, PlayerPrefs.GetInt("maxLevel"));
			}
		}

		if (gameStart) {
			if (Input.GetKeyUp (KeyCode.Escape))
				Application.Quit ();
		}
	}

	void OnApplicationPause(bool status) {

		// ToastAnalytics ////////////////////////////////////////////////////////
		// 앱이 Background/Foreground 이동시 처리해야 하는 부분입니다.
		// 앱이 Background로 들어갈때 traceDeactivation을 다시 Foreground로 이동할때 traceActivation을 호출해야 합니다.
		if (status == true) {
			GameAnalytics.traceDeactivation();
		} else {
			GameAnalytics.traceActivation();
		}
	}
}


// ToastAnalytics ////////////////////////////////////////////////////////
// Campaign Listener 구현체 입니다. CampaignListener Interface를 구현합니다.
// 각 callback에 대한 내용은 Programming Guide 참고하세요.
public class MyCampaignListener : CampaignListener {
	
	public void OnCampaignVisibilityChanged(string adspaceName, bool show) {
		Debug.Log ("OnCampaignVisibilityChanged : " + adspaceName + " / show : " + show);
	}
	
	public void OnCampaignLoadSuccess(string adspaceName) {
		Debug.Log ("OnCampaignLoadSuccess : " + adspaceName);
	}
	
	public void OnCampaignLoadFail(string adspaceName, int errorCode, string errorMessage) {
	}
	
	public void OnMissionComplete(List<string> missionList) {
		Debug.Log ("OnMissionComplete");
	}

	public void OnCampaignClick(string callbackInfo) {
		Debug.Log ("OnCampaignClick : " + callbackInfo);
	}
	
	public void OnPromotionVisibilityChanged(bool show) {
		
	}
}